"""
Dataclasses API for PyPI and other warehouses.

PyPI: https://pypi.org/
Warehouse: https://github.com/pypa/warehouse
"""
from __future__ import annotations

__version__ = '1.0.0b4'
__author__ = 'Misha Drachuk'
__license__ = 'MIT'

from datetime import datetime
from dataclasses import dataclass
from typing import Dict, Set, List, Optional

import requests
from pkg_resources import safe_version
from serious import JsonModel


def fetch_package(name: str, warehouse='https://pypi.org/pypi') -> Package:
    """Fetch a package from pypi.org by name.

    Provide a URL for other warehouse to use it instead of PyPI.
    """
    response = requests.get(f'{warehouse}/{name}/json')
    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError as e:
        raise PackageIndexError(name) from e
    package_json = response.text
    model = JsonModel(Package, camel_case=False)
    return model.load(package_json)


Version = str  # an alias for correct typings


@dataclass
class Package:
    info: PackageInfo
    releases: Dict[Version, List[Release]]
    last_serial: int
    urls: List[Release]

    @property
    def versions(self) -> Set[str]:
        return set(self.releases.keys())

    def contains_version(self, target: str) -> bool:
        target = safe_version(target)
        existing = {safe_version(version) for version in self.versions}
        return target in existing


@dataclass(frozen=True)
class PackageInfo:
    name: str
    author: str
    author_email: str
    bugtrack_url: Optional[str]
    classifiers: List[str]
    description: str
    description_content_type: str
    docs_url: Optional[str]
    download_url: str
    downloads: Dict[str, int]
    home_page: str
    keywords: str
    license: str
    maintainer: str
    maintainer_email: str
    package_url: str
    platform: str
    project_url: str
    project_urls: Dict[str, str]
    release_url: str
    requires_dist: Optional[List[str]]
    requires_python: str
    summary: str
    version: str


@dataclass(frozen=True)
class Release:
    comment_text: str
    digests: Dict[str, str]
    downloads: int
    filename: str
    has_sig: bool
    md5_digest: str
    packagetype: str
    python_version: str
    requires_python: str
    size: int
    upload_time: datetime
    url: str


class PackageIndexError(Exception):
    def __init__(self, name: str):
        super().__init__(f'Package "{name}" could not be fetched from PyPI. ')



from setuptools import setup

import package_index


def readme():
    with open('README.md', 'r', encoding='utf8') as f:
        return f.read()


setup(
    name='package-index',
    version=package_index.__version__,
    py_modules=['package_index'],
    author='mdrachuk',
    author_email='misha@drach.uk',
    description="Dataclasses API for PyPI and other warehouses.",
    long_description=readme(),
    long_description_content_type='text/markdown',
    url="https://github.com/mdrachuk/package-index",
    license="MIT",
    keywords="python packaging pypi warehouse",
    python_requires=">=3.7",
    zip_safe=False,
    project_urls={
        'Pipelines': 'https://dev.azure.com/misha-drachuk/package-index',
        'Source': 'https://github.com/mdrachuk/package-index/',
        'Issues': 'https://github.com/mdrachuk/package-index/issues',
    },
    install_requires=[
        'requests>=2.22.0',
        'serious==1.0.0.dev20',
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Topic :: Utilities",
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: POSIX",
        "Operating System :: POSIX :: BSD",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: Implementation :: CPython",
        "Typing :: Typed",
    ],
)
